@@ -22,11 +22,13 @@ Usage:
           - local: Non-distributed, possibly multiple if n > 1 (default)
           - ipython: IPython distributed processing
           - messaging: RabbitMQ distributed messaging queue
-     -n number of processes to start
-     -p profile name for ipython parallelization
+     -n total number of processes to use
+     -s scheduler for ipython parallelization (lsf, sge)
+     -q queue to submit jobs for ipython parallelization
 """
 import os
 import sys
+import subprocess
 
 from bcbio.pipeline.run_info import get_run_info
 from bcbio.distributed import manage as messaging
@@ -34,7 +36,7 @@ from bcbio.pipeline.config_utils import load_config
 from bcbio.pipeline.main import run_main, parse_cl_args
 
 def main(config_file, fc_dir=None, run_info_yaml=None, numcores=None,
-         paralleltype=None, profile="default"):
+         paralleltype=None, queue=None, scheduler=None, upgrade=None):
     work_dir = os.getcwd()
     config = load_config(config_file)
     if config.get("log_dir", None) is None:
@@ -42,7 +44,7 @@ def main(config_file, fc_dir=None, run_info_yaml=None, numcores=None,
     paralleltype, numcores = _get_cores_and_type(config, fc_dir, run_info_yaml,
                                                  numcores, paralleltype)
     parallel = {"type": paralleltype, "cores": numcores,
-                "profile": profile,
+                "scheduler": scheduler, "queue": queue,
                 "module": "bcbio.distributed"}
     if parallel["type"] in ["local", "messaging-main"]:
         if numcores is None:
@@ -54,8 +56,9 @@ def main(config_file, fc_dir=None, run_info_yaml=None, numcores=None,
         args = [config_file, fc_dir]
         if run_info_yaml:
             args.append(run_info_yaml)
-        messaging.run_and_monitor(config, config_file, args, parallel) 
+        messaging.run_and_monitor(config, config_file, args, parallel)
     elif parallel["type"] == "ipython":
+        assert parallel["queue"] is not None, "Ipython parallel requires a specified queue (-q)"
         run_main(config, config_file, work_dir, parallel,
                  fc_dir, run_info_yaml)
     else:
@@ -82,7 +85,7 @@ def _get_cores_and_type(config, fc_dir, run_info_yaml,
                 paralleltype = config_cores
     if paralleltype is None:
         paralleltype = "local"
-        
+
     if numcores is None:
         if config["distributed"].get("num_workers", "") == "all":
             cp = config["distributed"]["cluster_platform"]
@@ -104,6 +107,21 @@ def _needed_workers(run_info):
             names.append(x.get("name", (x["lane"], x["barcode_id"])))
     return len(set(names))
 
+def _upgrade_bcbio(method):
+    """Perform upgrade of bcbio to latest release, or from GitHub development version.
+    """
+    url = "https://raw.github.com/chapmanb/bcbio-nextgen/master/requirements.txt"
+    pip_bin = os.path.join(os.path.dirname(sys.executable), "pip")
+    if method in ["stable", "system"]:
+        sudo_cmd = [] if method == "stable" else ["sudo"]
+        subprocess.check_call(sudo_cmd + [pip_bin, "install", "--upgrade", "distribute"])
+        subprocess.check_call(sudo_cmd + [pip_bin, "install", "-r", url])
+    else:
+        raise NotImplementedError("Development upgrade")
+
 if __name__ == "__main__":
     config_file, kwargs = parse_cl_args(sys.argv[1:])
-    main(config_file, **kwargs)
+    if kwargs["upgrade"] and config_file is None:
+        _upgrade_bcbio(kwargs["upgrade"])
+    else:
+        main(config_file, **kwargs)
